/* Search v1.11 by Thorbjrn Andersson */

#include	<stdio.h>

#define		READ_BUFFER_SIZE 8192
#define		WORD_LENGTH		 256

#pragma export on
void 		process_refs(record_ref aReference, BMessage* msg, void*);
#pragma export reset

BListView	*EntryView;
BWindow		*ResultWindow;

bool		WindowInit(char *word);

bool 		ClipBoardSuccess( 	char 		*word 		);
bool 		Contains( 			char 		*buffer, 	
								char 		*word,
								long		buffer_length 	);

long 		ScanFile( 			BFile 		*aFile,
								char		*word,
								long 		word_len	);

long 		ScanDirectory( 		BDirectory 	*aDirectory,
								char		*word,
								long 		word_len);

bool
WindowInit(char *word)
{
	char			WinTitle[255];
	BAlert			*anAlert;

	sprintf(WinTitle, "Search result for %s", word);

	EntryView 	= new BListView( BRect(0,0,200,200),"Entries found", B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM );
	ResultWindow = new BWindow( 	BRect(60,20,260,220),WinTitle,B_TITLED_WINDOW,B_WILL_ACCEPT_FIRST_CLICK);
	
	if( EntryView == NULL )
	{
		anAlert = new BAlert("", "Unable to create a ListView.", "I'm sorry!");
		anAlert->Go();
		return false;
	}
	
	if( ResultWindow == NULL )
	{
		delete EntryView;
		anAlert = new BAlert("", "Unable to create a Window.", "I'm sorry!");
		anAlert->Go();
		return false;
	}
	
	ResultWindow->AddChild( EntryView );
	ResultWindow->Show();
	return true;
}

bool
ClipBoardSuccess( char *word)
{
	// Gets the search word from the clipboard.
	// false if it fails, true otherwise.
	
	long ClipLength;
	const char *tmpStr;
	
	be_clipboard->Lock();
	tmpStr = be_clipboard->FindText( &ClipLength );
	be_clipboard->Unlock();
	
	if( tmpStr!= NULL )
	{	
		//tmpStr just a pointer to clipboard data. Need to copy:
		strncpy( word, tmpStr, 	WORD_LENGTH );
		
		// Make sure we have null-terminated string..
		word[ min( WORD_LENGTH, ClipLength ) ] ='\0';	
						
		return true;
	}
	
	return false;
}

bool
Contains( char *buffer, char *word, long buffer_length )
{
	// Scans the buffer for the string.

	long i,j,k;
	for( i=0; i<buffer_length; i++)
	{
		j = 0; // Index for the searched word.
		k = i; // Temporary index for the buffer.
		
		while( buffer[ k ] == word[j] && word[j] != '\0' && k< buffer_length)
		{
			k++;
			j++;
		}	
		if( word[j] == '\0' )
			return true;
	}
	return false;
}

long
ScanFile(BFile *aFile, char *word, long word_len)
{
	// Scans the entire file for the string we are looking for.

	bool	WordFound = false;

	char 	*theName=NULL,
			theMessage[256],
			theReadBuffer[ READ_BUFFER_SIZE ];

	long	numOfBytesRead 	= READ_BUFFER_SIZE;

	BAlert 	*anAlert;

	if( aFile->Open( B_READ_ONLY ) == B_NO_ERROR )
	{
		while( numOfBytesRead == READ_BUFFER_SIZE && !WordFound )
		{
			numOfBytesRead = aFile->Read( theReadBuffer, READ_BUFFER_SIZE );

			if( Contains( theReadBuffer, word, numOfBytesRead ) == true )
				WordFound = true;
			// If we did NOT find the word in this buffer we must rewind
			// the filepointer to make sure the word does not lie in the
			// boundary of the two pages:
			else	
				aFile->Seek( -word_len, B_SEEK_MIDDLE );
		}

		if( WordFound == true )
		{
			if( (modifiers() & B_SHIFT_KEY) == 0)
			{		
				theName = new char[350];
				if( aFile->GetPath( theName, 349 ) == B_ERROR )
					aFile->GetName( theName );
			}
			else
			{
				theName = new char[64];
				aFile->GetName( theName );
			}
			ResultWindow->Lock();
			EntryView->AddItem( theName );
			ResultWindow->Unlock();
		}

		aFile->Close();

		return 0;
	}
	else
	{
		aFile->GetName( theName );
		sprintf( theMessage, "Could not open '%s'", theName);
		anAlert = new BAlert("", theMessage, "Continue","Stop");
		return anAlert->Go();
	}
	
	return 1;	//If we end up here we want to stop! Somthin's weeeiiirrdd!
}

long
ScanDirectory(BDirectory *aDirectory, char *word, long word_len)
{
	// Scans the directories recursively down for all files and folders.
	// Examines every file.

	long		i,
				CallForBreak = 0,
				NumberOfFiles, 
				NumberOfDirs;

	BFile		theFile;

	BDirectory	theDirectory;
	
	NumberOfFiles 	= aDirectory->CountFiles();
	NumberOfDirs 	= aDirectory->CountDirectories();

	for( i=0; i<NumberOfFiles && !CallForBreak; i++)
	{
		aDirectory->GetFile( i, &theFile );

		if( theFile.Error() == B_NO_ERROR )
			CallForBreak = ScanFile( &theFile, word, word_len );
	}

	for( i=0; i<NumberOfDirs && !CallForBreak; i++)
	{
		aDirectory->GetDirectory( i, &theDirectory );
		
		if( theDirectory.Error() == B_NO_ERROR )
			CallForBreak = ScanDirectory( &theDirectory, word, word_len );
	}
	
	return CallForBreak;
}

void
process_refs(record_ref, BMessage* msg, void*)
{
	char			word[ WORD_LENGTH ];
	long			i, numOfRefs,
					CallForBreak = 0,
					WordLength;

	ulong			type;

	BFile			theFile;
	BDirectory		theDirectory;
	record_ref		theReference;

	BAlert			*anAlert;
		
	msg->GetInfo("refs", &type, &numOfRefs);
	
	if( ClipBoardSuccess( word ) == false )
	{
		anAlert = new BAlert("", "Nothing to search for: Clipboard is empty.", "Got that.");
		anAlert->Go();
		return;
	}

	WordLength = strlen( word );

	if (numOfRefs == 0)
	{
		anAlert = new BAlert("", "No valid selection found.", "Forgot that.");
		anAlert->Go();
		return;
	}
		
	if( WindowInit( word ) == false )
		return;
		
	for( i = 0; i<numOfRefs && !CallForBreak; i++)
	{
		theReference = msg->FindRef("refs", i);
		
		if( does_ref_conform( theReference, "File" ))
		{
			theFile.SetRef( theReference );
		
			if( theFile.Error() == B_NO_ERROR )
				CallForBreak = ScanFile( &theFile, word, WordLength );
		}
		else
		{
			theDirectory.SetRef( theReference );
	
			if( theDirectory.Error() == B_NO_ERROR )
				CallForBreak = ScanDirectory( &theDirectory, word, WordLength );
		}
	}
}